#!/bin/bash

calculate_date_difference() {
  # Check if the certificate file path is provided
  if [ -z "$1" ]; then
    echo "Usage: $0 <cert_file>"
    return 1
  fi

  CERT_FILE="$1"
  # Extract the Not Before date from the certificate file
  NOT_BEFORE_DATE=$(openssl x509 -in $CERT_FILE -noout -startdate | sed 's/notBefore=//')
  # Add 24 hours to the extracted date
  NOT_BEFORE_DATE_PLUS_24=$(date -d "$NOT_BEFORE_DATE 24 hours" +"%Y-%m-%d %H:%M:%S")
  # Get the current date
  CURRENT_DATE=$(date +"%Y-%m-%d %H:%M:%S")
  # Convert both dates to epoch time
  NOT_BEFORE_DATE_EPOCH=$(date -d "$NOT_BEFORE_DATE_PLUS_24" +%s)
  CURRENT_DATE_EPOCH=$(date -d "$CURRENT_DATE" +%s)
  # Calculate the difference in seconds
  DIFFERENCE_SECONDS=$((CURRENT_DATE_EPOCH - NOT_BEFORE_DATE_EPOCH))
  # Convert the difference to hours
  DIFFERENCE_HOURS=$((DIFFERENCE_SECONDS / 3600))

  # Output the result
  echo $DIFFERENCE_HOURS

  # Return success status
  return 0
}

extract_CA_Certs() {
  ORG_NAME=org-orderer && \
  CA_NAME=${ORG_NAME}-ca && \
  CA_DIR=${TEMP_DIR}/cas/${CA_NAME} && \
  mkdir -p $CA_DIR && \
  kubectl -n consip-fabric-namespace get secret ${CA_NAME}-tls-cert -o json \
  | jq -r .data.\"ca.crt\" \
  | base64 -d > ${CA_DIR}/tlsca-cert.pem

  # TODO: Use right port for Consip Environments
  ORG_MSP_DIR=${TEMP_DIR}/channel-msp/ordererOrganizations/${ORG_NAME}/msp
  mkdir -p ${ORG_MSP_DIR}/cacerts
  mkdir -p ${ORG_MSP_DIR}/tlscacerts
  curl -s \
  --cacert ${TEMP_DIR}/cas/${CA_NAME}/tlsca-cert.pem \
  https://$CA_NAME$BASE_ENDPOINT/cainfo \
  | jq -r .result.CAChain \
  | base64 -d > ${ORG_MSP_DIR}/cacerts/ca-signcert.pem

  kubectl -n consip-fabric-namespace get secret ${CA_NAME}-tls-cert -o json \
  | jq -r .data.\"ca.crt\" \
  | base64 -d > ${ORG_MSP_DIR}/tlscacerts/tlsca-signcert.pem

  ORG_NAME=org-consip && \
  CA_NAME=${ORG_NAME}-ca && \
  CA_DIR=${TEMP_DIR}/cas/${CA_NAME} && \
  mkdir -p $CA_DIR && \
  kubectl -n consip-fabric-namespace get secret ${CA_NAME}-tls-cert -o json \
  | jq -r .data.\"ca.crt\" \
  | base64 -d > ${CA_DIR}/tlsca-cert.pem
  # TODO: Use right port for Consip Environments
  ORG_MSP_DIR=${TEMP_DIR}/channel-msp/peerOrganizations/${ORG_NAME}/msp
  mkdir -p ${ORG_MSP_DIR}/cacerts
  mkdir -p ${ORG_MSP_DIR}/tlscacerts
  curl -s \
  --cacert ${TEMP_DIR}/cas/${CA_NAME}/tlsca-cert.pem \
  https://$CA_NAME$BASE_ENDPOINT/cainfo \
  | jq -r .result.CAChain \
  | base64 -d > ${ORG_MSP_DIR}/cacerts/ca-signcert.pem

  kubectl -n consip-fabric-namespace get secret ${CA_NAME}-tls-cert -o json \
  | jq -r .data.\"ca.crt\" \
  | base64 -d > ${ORG_MSP_DIR}/tlscacerts/tlsca-signcert.pem
}

extract_Orderer_TLS_Certs() {
  for i in "${ORDERER_INDEXES[@]}"; do
    ORDERER_INDEX=$i
    ORG=org-orderer
    ORDERER_TLS_DIR=${TEMP_DIR}/channel-msp/ordererOrganizations/${ORG}/orderers/${ORG}-orderer${ORDERER_INDEX}/tls
    mkdir -p $ORDERER_TLS_DIR/signcerts

    kubectl -n consip-fabric-namespace get secret ${ORG}-orderer${ORDERER_INDEX}-tls-cert -o json \
    | jq -r .data.\"tls.crt\" \
    | base64 -d > ${ORDERER_TLS_DIR}/signcerts/tls-cert.pem
  done

}

reenroll_AdminRootCA() {
  blackbox_decrypt_all_files

  # TODO: Use right port for Consip Environments
  CA_NAME=org-orderer-ca && \
  CA_DIR=${TEMP_DIR}/cas/${CA_NAME} && \
  USER=org-orderer-root-ca-admin && \
  PSW=$(grep -A50 Orderers USERS.md | grep -A30 "Root CA Admin" | grep -A4 "$(echo " ${ENV}" | tr '[:lower:]' '[:upper:]')" | grep Password | grep -Po '`([-a-z0-9]+)`$' | tr -d '`') && \
  ../bin/fabric-ca-client reenroll \
  --url                   https://$USER:$PSW@org-orderer-ca$BASE_ENDPOINT \
  --tls.certfiles         $CA_DIR/tlsca-cert.pem \
  --csr.keyrequest.reusekey \
  --mspdir                $TEMP_DIR/enrollments/org-orderer/users/${USER}/msp

  # TODO: Use right port for Consip Environments
  CA_NAME=org-consip-ca && \
  CA_DIR=${TEMP_DIR}/cas/${CA_NAME} && \
  USER=org-consip-root-ca-admin && \
  PSW=$(grep -A50 Peers USERS.md | grep -A30 "Root CA Admin" | grep -A4 "$(echo " ${ENV}" | tr '[:lower:]' '[:upper:]')" | grep Password | grep -Po '`([-a-z0-9]+)`$' | tr -d '`') && \
  ../bin/fabric-ca-client reenroll \
  --url                   https://$USER:$PSW@org-consip-ca$BASE_ENDPOINT \
  --tls.certfiles         $CA_DIR/tlsca-cert.pem \
  --csr.keyrequest.reusekey \
  --mspdir                $TEMP_DIR/enrollments/org-consip/users/${USER}/msp
}

reenroll_Orderers() {

  for i in "${ORDERER_INDEXES[@]}"; do
    ORDERER_INDEX=$i && \
    PSW=$(grep -A100 Orderers USERS.md | grep -A5 "Users" | grep Password | grep -Po '`([-a-z]+)`$' | tr -d '`') && \
    MSP_DIR=/var/hyperledger/fabric/organizations/ordererOrganizations/org-orderer.fabric.com/orderers/orderer$ORDERER_INDEX.org-orderer.fabric.com/msp && \
    cat <<EOF | kubectl -n consip-fabric-namespace exec deploy/org-orderer-ca -i -- /bin/sh

set -x
export FABRIC_CA_CLIENT_HOME=/var/hyperledger/fabric-ca-client
export FABRIC_CA_CLIENT_TLS_CERTFILES=/var/hyperledger/fabric/config/tls/ca.crt

fabric-ca-client reenroll \
--url https://orderer$ORDERER_INDEX:$PSW@org-orderer-ca \
--csr.hosts orderer$ORDERER_INDEX \
--csr.keyrequest.reusekey \
--mspdir $MSP_DIR
EOF
  done
}

reenroll_Peers() {
  for i in "${PEER_INDEXES[@]}"; do
    PEER_INDEX=$i && \
    PSW=$(grep -A100 Peers USERS.md | grep -A5 "Users" | grep Password | grep -Po '`([-a-z]+)`$' | tr -d '`') && \
    MSP_DIR=/var/hyperledger/fabric/organizations/peerOrganizations/org-consip.fabric.com/peers/peer$PEER_INDEX.org-consip.fabric.com/msp && \
    cat <<EOF | kubectl -n consip-fabric-namespace exec deploy/org-consip-ca -i -- /bin/sh

  set -x
export FABRIC_CA_CLIENT_HOME=/var/hyperledger/fabric-ca-client
export FABRIC_CA_CLIENT_TLS_CERTFILES=/var/hyperledger/fabric/config/tls/ca.crt

fabric-ca-client reenroll \
--url https://peer$PEER_INDEX:$PSW@org-consip-ca \
--csr.hosts localhost,org-consip-peer$PEER_INDEX,org-consip-peers-service \
--csr.keyrequest.reusekey \
--mspdir $MSP_DIR
EOF
  done
}

reenroll_OrgAdmin() {
  blackbox_decrypt_all_files

  # TODO: Use right port for Consip Environments
  TEMP_DIR=${PWD}/build/${ENV} && \
  USER=org-orderer-admin && \
  PSW=$(grep -A70 Orderers USERS.md | grep -A30 "Org Admin" | grep -A5 "$(echo " ${ENV}" | tr '[:lower:]' '[:upper:]')" | grep Password | grep -Po '`([-a-z0-9]+)`$' | tr -d '`') && \
  ORG_ADMIN_DIR=${TEMP_DIR}/enrollments/org-orderer/users/${USER} && \
  CA_DIR=${TEMP_DIR}/cas/org-orderer-ca && \
  export FABRIC_CA_CLIENT_HOME=${ORG_ADMIN_DIR} && \
  ../bin/fabric-ca-client reenroll \
  --url https://$USER:$PSW@org-orderer-ca$BASE_ENDPOINT \
  --tls.certfiles ${CA_DIR}/tlsca-cert.pem \
  --csr.keyrequest.reusekey \
  --mspdir ${TEMP_DIR}/enrollments/org-orderer/users/${USER}/msp

  # TODO: Use right port for Consip Environments
  TEMP_DIR=${PWD}/build/${ENV} && \
  USER=org-consip-admin && \
  PSW=$(grep -A70 Peers USERS.md | grep -A30 "Org Admin" | grep -A5 "$(echo " ${ENV}" | tr '[:lower:]' '[:upper:]')" | grep Password | grep -Po '`([-a-z0-9]+)`$' | tr -d '`') && \
  ORG_ADMIN_DIR=${TEMP_DIR}/enrollments/org-consip/users/${USER} && \
  CA_DIR=${TEMP_DIR}/cas/org-consip-ca && \
  export FABRIC_CA_CLIENT_HOME=${ORG_ADMIN_DIR} && \
  ../bin/fabric-ca-client reenroll \
  --url https://$USER:$PSW@org-consip-ca$BASE_ENDPOINT \
  --tls.certfiles ${CA_DIR}/tlsca-cert.pem \
  --csr.keyrequest.reusekey \
  --mspdir ${TEMP_DIR}/enrollments/org-consip/users/${USER}/msp
}

reenroll_API_Rest_Client() {
  blackbox_decrypt_all_files

  # TODO: Use right port for Consip Environments
  TEMP_DIR=${PWD}/build/${ENV} && \
  USER=api-rest-client && \
  PSW=$(grep -A20 Clients USERS.md | grep -A5 "API Rest Client" | grep Password | grep -Po '`([-a-z]+)`$' | tr -d '`') && \
  ORG_CLIENT_DIR=${TEMP_DIR}/enrollments/org-consip/users/${USER} && \
  CA_DIR=${TEMP_DIR}/cas/org-consip-ca && \
  export FABRIC_CA_CLIENT_HOME=${ORG_CLIENT_DIR} && \
  ../bin/fabric-ca-client reenroll \
  --url https://$USER:$PSW@org-consip-ca$BASE_ENDPOINT \
  --tls.certfiles ${CA_DIR}/tlsca-cert.pem \
  --csr.keyrequest.reusekey \
  --mspdir ${TEMP_DIR}/enrollments/org-consip/users/${USER}/msp
}

update_Secrets() {
  blackbox_decrypt_all_files

  K8S_CONFIG_FILE=fabric-blockchain/$ENV/explorer/hyperledger-explorer-admin-secret.yaml
  cat << EOF > ${K8S_CONFIG_FILE}
apiVersion: v1
kind: Secret
metadata:
  name: hyperledger-explorer-admin-secret
  namespace: consip-fabric-namespace
  labels:
    app: hyperledger-explorer
    service: explorer
type: kubernetes.io/tls
data:
  tls.crt: ADMIN_TLS_CRT
  tls.key: ADMIN_TLS_KEY
EOF

  ADMIN_TLS_CRT=$(base64 build/$ENV/enrollments/org-consip/users/org-consip-admin/msp/signcerts/cert.pem)
  ADMIN_TLS_KEY=$(base64 build/$ENV/enrollments/org-consip/users/org-consip-admin/msp/keystore/key.pem)

  ADMIN_TLS_CRT=$(echo "$ADMIN_TLS_CRT" | tr -d '\n')
  ADMIN_TLS_KEY=$(echo "$ADMIN_TLS_KEY" | tr -d '\n')

  sed -i "s/ADMIN_TLS_CRT/${ADMIN_TLS_CRT}/" "$K8S_CONFIG_FILE"
  sed -i "s/ADMIN_TLS_KEY/${ADMIN_TLS_KEY}/" "$K8S_CONFIG_FILE"

  kubectl apply -f $K8S_CONFIG_FILE

  rm $K8S_CONFIG_FILE

  # TODO: uncomment
#  USER=api-rest-client && \
#  K8S_CONFIG_FILE=fabric-blockchain/$ENV/api-rest-client-secret.yaml
#  cat << EOF > ${K8S_CONFIG_FILE}
#apiVersion: v1
#kind: Secret
#metadata:
#  name: api-rest-client-secret
#  namespace: consip-api-rest
#  labels:
#    org: org-consip
#    app: api-rest
#type: kubernetes.io/tls
#data:
#  tls.crt: TLS_CRT
#  tls.key: TLS_KEY
#EOF
#
#  TLS_CRT=$(base64 build/$ENV/enrollments/org-consip/users/${USER}/msp/signcerts/cert.pem)
#  TLS_KEY=$(base64 build/$ENV/enrollments/org-consip/users/${USER}/msp/keystore/key.pem)
#
#  TLS_CRT=$(echo "$TLS_CRT" | tr -d '\n')
#  TLS_KEY=$(echo "$TLS_KEY" | tr -d '\n')
#
#  sed -i "s/TLS_CRT/${TLS_CRT}/" "$K8S_CONFIG_FILE"
#  sed -i "s/TLS_KEY/${TLS_KEY}/" "$K8S_CONFIG_FILE"
#
#  kubectl apply -f $K8S_CONFIG_FILE
#
#  rm $K8S_CONFIG_FILE
}

restartOrderers() {
  # Orderers
  kubectl delete -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer0-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer1-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer2-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer3-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer4-deployment.yaml

  sleep 10

  kubectl apply -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer0-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer1-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer2-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer3-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-orderer/org-orderer-orderer4-deployment.yaml

  sleep 20
}

restartPeers() {
  # Peers
  kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer0-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer1-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer2-deployment.yaml
  kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer3-deployment.yaml
  if [ "$ENV" != "" ] && [ "$ENV" != "dev" ]; then
    kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer4-deployment.yaml
    kubectl delete -f fabric-blockchain/${ENV}/org-consip/org-consip-peer5-deployment.yaml
  fi
  sleep 10
  kubectl -n consip-fabric-namespace delete pod -l "app.kubernetes.io/component=chaincode" --field-selector=status.phase=Succeeded
  kubectl -n consip-fabric-namespace delete secret -l "app.kubernetes.io/component=chaincode"

  sleep 10

  kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer0-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer1-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer2-deployment.yaml
  kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer3-deployment.yaml
  if [ "$ENV" != "" ] && [ "$ENV" != "dev" ]; then
    kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer4-deployment.yaml
    kubectl apply -f fabric-blockchain/${ENV}/org-consip/org-consip-peer5-deployment.yaml
  fi

  sleep 20
}

restartExplorer() {
   kubectl -n consip-fabric-namespace rollout restart deployment/hyperledger-explorer-deployment
}

restartServices() {
  restartOrderers

  sleep 20

  restartPeers

  sleep 10

  restartExplorer
}

redeployHyperledgerExplorer() {
  kubectl delete -f fabric-blockchain/${ENV}/explorer/hyperledger-explorer.yaml
  kubectl delete -f fabric-blockchain/${ENV}/explorer/hyperledger-explorer-persistence.yaml

  sleep 20

  kubectl apply -f fabric-blockchain/${ENV}/explorer/hyperledger-explorer-persistence.yaml
  kubectl apply -f fabric-blockchain/${ENV}/explorer/hyperledger-explorer.yaml
}

submitChannelUpdates() {
  PEER_ORG=org-consip
  PEER_INDEX=0
  PEER_PORT=30751
  export CHANNEL=consip-channel
  echo "Channel: $CHANNEL"

  FETCHED_CONFIG_PB=blockFetchedConfig.pb
  CURRENT_CONFIG_PB=config-block.pb
  CURRENT_CONFIG_JSON=config-block.json
  CONFIG_CHANGES_JSON=config-changes.json
  CONFIG_PROPOSAL_PB=config-proposal.pb
  CONFIG_PROPOSAL_JSON=config-proposal.json
  CONFIG_SUBMIT_READY_PB=config-submit-ready.pb
  CONFIG_SUBMIT_READY_JSON=config-submit-ready.json

  ORDERER_ORG=org-orderer
  ORG_CHANNEL_BASE_DIR=${TEMP_DIR}/channel-msp/ordererOrganizations/${ORDERER_ORG}
  ORG_MSP_DIR=${ORG_CHANNEL_BASE_DIR}/msp

  # Modify the configuration file
  for i in "${ORDERER_INDEXES[@]}"; do

    export FABRIC_CFG_PATH=${BASE_PATH}/../config/${PEER_ORG}
    export CORE_PEER_MSPCONFIGPATH=${TEMP_DIR}/enrollments/${PEER_ORG}/users/${PEER_ORG}-admin/msp
    export CORE_PEER_TLS_ROOTCERT_FILE=${TEMP_DIR}/channel-msp/peerOrganizations/${PEER_ORG}/msp/tlscacerts/tlsca-signcert.pem
    export CORE_PEER_ADDRESS=${PEER_ORG}-peer${PEER_INDEX}${BASE_ENDPOINT}:${PEER_PORT}
    echo "PeerAddress: $CORE_PEER_ADDRESS"

    ORDERER_INDEX=$i
    echo "OrdererIndex: $ORDERER_INDEX"

    ORDERER_ADDRESS=${ORDERER_ORG}-orderer${i}${BASE_ENDPOINT}:${ORDERER_PORTS[$i]}
    echo "OrdererAddress: $ORDERER_ADDRESS"

    export ORDERER_TLS_DIR=${ORG_CHANNEL_BASE_DIR}/orderers/${ORDERER_ORG}-orderer${ORDERER_INDEX}/tls

    # Get the last configuration definition from the network
    ${BASE_PATH}/../bin/peer channel fetch config $FETCHED_CONFIG_PB -c $CHANNEL

    # Decode the fetched file and write into a JSON
    ${BASE_PATH}/../bin/configtxlator proto_decode --input $FETCHED_CONFIG_PB --type=common.Block | jq '.data.data[0].payload.data.config' > $CURRENT_CONFIG_JSON

    CONFIG_CHANGES_ORDERER_PB=config-changes-orderer${ORDERER_INDEX}.pb
    CONFIG_CHANGES_ORDERER_JSON=config-changes-orderer${ORDERER_INDEX}.json

    ORDERER_CERT_BASE64=$(cat ${ORDERER_TLS_DIR}/signcerts/tls-cert.pem | base64 -w 0)

    cat $CURRENT_CONFIG_JSON | \
    jq ".channel_group.groups.Orderer.values.ConsensusType.value.metadata.consenters[${ORDERER_INDEX}].server_tls_cert=\"${ORDERER_CERT_BASE64}\" | .channel_group.groups.Orderer.values.ConsensusType.value.metadata.consenters[${ORDERER_INDEX}].client_tls_cert=\"${ORDERER_CERT_BASE64}\"" > $CONFIG_CHANGES_ORDERER_JSON
#    cat $CONFIG_CHANGES_JSON | jq ".channel_group.groups.Orderer.values.ConsensusType.value.metadata.consenters[${ORDERER_INDEX}].server_tls_cert=\"${ORDERER_CERT_BASE64}\"" > "${CONFIG_CHANGES_ORDERER_JSON}"

#    NEW_BATCH_TIMEOUT=$((i + 1))
#    cat $CONFIG_CHANGES_JSON | jq ".channel_group.groups.Orderer.values.BatchTimeout.value.timeout=\"${NEW_BATCH_TIMEOUT}s\"" > $CONFIG_CHANGES_ORDERER_JSON

    # Encode the original configuration file to proto buffer
    ${BASE_PATH}/../bin/configtxlator proto_encode --input $CURRENT_CONFIG_JSON --type=common.Config --output $CURRENT_CONFIG_PB
    # Encode the file with the configuration changes to proto buffer
    ${BASE_PATH}/../bin/configtxlator proto_encode --input $CONFIG_CHANGES_ORDERER_JSON --type=common.Config --output $CONFIG_CHANGES_ORDERER_PB
    # Compute the modifications between the original file and the modified one
    ${BASE_PATH}/../bin/configtxlator compute_update --channel_id $CHANNEL --original $CURRENT_CONFIG_PB --updated $CONFIG_CHANGES_ORDERER_PB --output $CONFIG_PROPOSAL_PB
    # Decode the proto buffer file to a JSON
    ${BASE_PATH}/../bin/configtxlator proto_decode --input=$CONFIG_PROPOSAL_PB --type=common.ConfigUpdate --output=$CONFIG_PROPOSAL_JSON
    # Generate the JSON file containing the submit proposal for network update
    echo "{\"payload\":{\"header\":{\"channel_header\":{\"channel_id\":\"$CHANNEL\",\"type\":2}},\"data\":{\"config_update\":$(cat $CONFIG_PROPOSAL_JSON)}}}" | jq . > $CONFIG_SUBMIT_READY_JSON
    # Re-encode the JSON file to submit it to the network
    ${BASE_PATH}/../bin/configtxlator proto_encode --input $CONFIG_SUBMIT_READY_JSON --type=common.Envelope --output $CONFIG_SUBMIT_READY_PB

    # Orderer
    export CORE_PEER_LOCALMSPID=OrgOrdererMSP
    export CORE_PEER_TLS_ROOTCERT_FILE=${ORG_MSP_DIR}/cacerts/ca-signcert.pem
    export CORE_PEER_MSPCONFIGPATH=${TEMP_DIR}/enrollments/${ORDERER_ORG}/users/${ORDERER_ORG}-admin/msp
    export CORE_PEER_ADDRESS=${ORDERER_ADDRESS}

    # Sign the proto buffer file representing the new network configuration update
    ${BASE_PATH}/../bin/peer channel signconfigtx -f $CONFIG_SUBMIT_READY_PB

    export ORDERER_CA=${ORG_MSP_DIR}/tlscacerts/tlsca-signcert.pem
#    DIFFERENCE_HOURS=$(calculate_date_difference "$ORDERER_CA")

#    # Finally, send the update to the network for approval
##    ${BASE_PATH}/../bin/peer channel update -f $CONFIG_SUBMIT_READY_PB -c $CHANNEL -o $ORDERER_ADDRESS --tls --cafile $ORDERER_CA --tlsHandshakeTimeShift 2192h
#    ${BASE_PATH}/../bin/peer channel update -f $CONFIG_SUBMIT_READY_PB -c $CHANNEL -o $ORDERER_ADDRESS --tls --cafile $ORDERER_CA --tlsHandshakeTimeShift "${DIFFERENCE_HOURS}h"
    ${BASE_PATH}/../bin/peer channel update -f $CONFIG_SUBMIT_READY_PB -c $CHANNEL -o $ORDERER_ADDRESS --tls --cafile $ORDERER_CA

    unset ORDERER_TLS_DIR
    unset CORE_PEER_LOCALMSPID
    unset CORE_PEER_TLS_ROOTCERT_FILE
    unset CORE_PEER_MSPCONFIGPATH
    unset CORE_PEER_ADDRESS
    unset ORDERER_CA

    rm $CONFIG_CHANGES_ORDERER_PB
    rm $CONFIG_CHANGES_ORDERER_JSON

    sleep 5

  done

  rm $FETCHED_CONFIG_PB
  rm $CURRENT_CONFIG_PB
  rm $CURRENT_CONFIG_JSON
  rm $CONFIG_CHANGES_JSON
  rm $CONFIG_PROPOSAL_PB
  rm $CONFIG_PROPOSAL_JSON
  rm $CONFIG_SUBMIT_READY_PB
  rm $CONFIG_SUBMIT_READY_JSON
}

# Peer name indexes
export PEER_INDEXES=(0 1 2 3)

#export ORDERER_INDEXES=(0 1 2 3 4)
export ORDERER_INDEXES=(0)
export ORDERER_PORTS=(30650 30651 30652 30653 30654)

COMMON_DOMAIN=.acquistinretepa.it
case $ENV in
  "coll")
    export BASE_ENDPOINT="-collaudo$COMMON_DOMAIN"
    ;;
  "pre-prod")
    export BASE_ENDPOINT="-preesercizio$COMMON_DOMAIN"
    ;;
  "prod")
    export BASE_ENDPOINT="-esercizio$COMMON_DOMAIN"
    ;;
  "demo")
    export BASE_ENDPOINT="-demo$COMMON_DOMAIN"
    ;;
  "evo")
    export BASE_ENDPOINT="-evo$COMMON_DOMAIN"
    ;;
  *)
    export ENV=dev
    export BASE_ENDPOINT=.consip-lb-dev.it
    ;;
esac

BASE_PATH="$(pwd)"
echo "BasePath: ${BASE_PATH}"

export TEMP_DIR=${BASE_PATH}/build/${ENV}

#extract_CA_Certs
extract_Orderer_TLS_Certs
#reenroll_AdminRootCA
#reenroll_Orderers
#reenroll_Peers
#reenroll_OrgAdmin
##reenroll_API_Rest_Client
#update_Secrets
#restartServices
submitChannelUpdates
#redeployHyperledgerExplorer
